<?php

namespace WPIS;

class PostTypes {

    use EstatePostMetas;

    static public $postTypeName = 'wpis_estates';
    static public $postTypeMetaPrefix = 'wpis';

    private $enableTaxonomies = false;
    private $taxonomies = [
        [
            'slug'         => 'estate_cities',
            'single_name'  => 'Ville',
            'plural_name'  => 'Villes',
            'rewrite'      => ['slug' => 'city'],
        ],
        [
            'slug'         => 'estate_categories',
            'single_name'  => 'Catégorie',
            'plural_name'  => 'Catégories',
            'rewrite'      => ['slug' => 'category'],
        ],
        [
            'slug'         => 'estate_purposes',
            'single_name'  => 'Type de transaction',
            'plural_name'  => 'Types de transaction',
            'rewrite'      => ['slug' => 'type'],
        ],
        [
            'slug'         => 'estate_ammeneties',
            'single_name'  => 'Équipement',
            'plural_name'  => 'Équipements',
            'rewrite'      => ['slug' => 'ameneties'],
        ]
    ];

    private $enableImportSystem = false;
    private $enableImportCustomPostType = false;
    private $customPostTypeName = null;

	static public function init() {
        $class = new PostTypes();

        $class->wp_db = $GLOBALS['wpdb'];
		$class->wp_table_settings = $class->wp_db->prefix . 'wpis_settings';

        $class->checkSettings();

        if( $class->enableImportSystem && !$class->enableImportCustomPostType ){
            add_action( 'init', [$class, 'registerPostTypes']);
            add_action( 'add_meta_boxes', [$class, 'wpisAddMetabox']);
            add_action( 'save_post', [$class, 'wpisMetaboxSave'], 10, 2);
            add_filter( 'add_menu_classes', [$class, 'displayPostNumber']);
            add_filter( 'manage_' . self::$postTypeName .  '_posts_columns', [$class, 'setCustomColumns'] );
            add_action( 'manage_' . self::$postTypeName .  '_posts_custom_column' , [$class, 'applyCustomColumns'], 10, 2 );
        }

        if( $class->enableTaxonomies ){
            add_action( 'init' , [$class, 'registerTaxonomies'] );
        }

        return $class;
    }

    private function checkSettings(){
        $result = $this->wp_db->get_results("SELECT name, value FROM $this->wp_table_settings WHERE name = 'enableImportSystem'");
        if( !empty($result) && intval($result[0]->value) === 1 )
            $this->enableImportSystem = true;

        $result = $this->wp_db->get_results("SELECT name, value FROM $this->wp_table_settings WHERE name = 'enableImportCustomPostType'");
        if( !empty($result) && intval($result[0]->value) === 1 )
            $this->enableImportCustomPostType = true;

        $result = $this->wp_db->get_results("SELECT name, value FROM $this->wp_table_settings WHERE name = 'enableTaxonomies'");
        if( !empty($result) && intval($result[0]->value) === 1 )
            $this->enableTaxonomies = true;

        if( $this->enableImportCustomPostType ){
            $result = $this->wp_db->get_results("SELECT name, value FROM $this->wp_table_settings WHERE name = 'customPostType'");
            if( !empty($result) ){
                $this->customPostTypeName = $result[0]->value;
            }
        }
    }

    public function displayPostNumber( $menu ) {
        $postNumber = wp_count_posts(  PostTypes::$postTypeName, 'readable' );
        $postNumber = $postNumber->publish;

        foreach($menu as $index => $item){
            if( $item[2] === 'edit.php?post_type=' . PostTypes::$postTypeName){
                $style =
                "display: inline-block;" .
                "vertical-align: top;" .
                "margin: 0px 10px 0px 0px;" .
                "padding: 0 8px;" .
                "min-width: 7px;" .
                "height: 20px;" .
                "border-radius: 11px;" .
                "background-color: #00b9eb;" .
                "color: #fff;" .
                "line-height: 20px;" .
                "text-align: center;" .
                "z-index: 26;" .
                "float: right;" .
                "font-size: 12px;";
                $item[0] .= '<span style="' . $style .'">' . $postNumber . '</span>' ;
                $menu[$index] = $item;
            }
        }

        return $menu;
    }

    public function registerPostTypes(){
        register_post_type(self::$postTypeName,
            [
                'labels'      => [
                    'name'          => __('Propriétés'),
                    'singular_name' => __('Propriété'),
                ],
                'public'      => true,
                'has_archive' => true,
                'menu_icon'   => 'dashicons-admin-home',
                // 'show_in_rest' => true,
                'supports' => array( 'title', 'editor', 'thumbnail' )
            ]
        );

    }

    public function setCustomColumns($columns) {

        $columns['thumbnail'] = __( 'Image', 'wpis' );
        $columns['type-category'] = __( 'Catégorie', 'wpis' );
        $columns['price'] = __( 'Prix', 'wpis' );
        $columns['purpose'] = __( 'Status', 'wpis' );
        $columns['reference'] = __( 'Référence', 'wpis' );
        $columns['locked'] = __( 'Verrouillé', 'wpis' );

        $columnClone = $columns;

        $order = [
            'cb',
            'thumbnail',
            'title',
            'type-category',
            'price',
            'purpose',
            'reference',
            'locked',
            'date',
        ];

        usort($columns, function ($a, $b) use ($order) {
            if( isset($columns) ){
                $pos_a = array_search($columns[$a], $order);
                $pos_b = array_search($columns[$b], $order);
                return $pos_a - $pos_b;
            }
        });

        $columnsMapped = [];

        foreach( $order as $i => $value ){
            $columnsMapped[$order[$i]] = $columnClone[$value];
        }

        return $columnsMapped;
    }

    public function applyCustomColumns( $column, $post_id ) {
        switch ( $column ) {

            case 'thumbnail' :
                echo get_the_post_thumbnail( $post_id, [50, 50] );
                break;

            case 'type-category' :
                $type = get_post_meta( $post_id , self::$postTypeMetaPrefix . '_' . 'category_label' , true );
                $category = get_post_meta( $post_id , self::$postTypeMetaPrefix . '_' . 'subcategory_label' , true );
                $typeCategory = self::ucfirstUtf8($type) . ' / ' . self::ucfirstUtf8($category);
                echo $typeCategory;
                break;

            case 'price' :
                $price = round(get_post_meta( $post_id , self::$postTypeMetaPrefix . '_' . 'finance_price' , true ));
                $price = number_format_i18n($price);
                $price = str_replace(',', ' ', $price);
                $currency = get_post_meta( $post_id , self::$postTypeMetaPrefix . '_' . 'finance_currency' , true );
                echo $price . ' '  . $currency;
                break;

            case 'purpose' :
                $purpose = get_post_meta( $post_id , self::$postTypeMetaPrefix . '_' . 'purpose_label' , true );
                $purpose = self::ucfirstUtf8($purpose);
                echo $purpose;
                break;

            case 'reference' :
                $reference = get_post_meta( $post_id , self::$postTypeMetaPrefix . '_' . 'reference' , true );
                echo $reference;
                break;

            case 'locked' :
                $isLocked = get_post_meta( $post_id , self::$postTypeMetaPrefix . '_' . 'locked' , true );
                echo $isLocked ? '<i class="fas fa-lock"></i>' : '<i class="fas fa-lock-open"></i>';
                break;
        }
    }

    public function wpisAddMetabox(){

        // Estate Properties metabox
        foreach(EstatePostMetas::$postTypeMetas as $group) {
            if( $group['id'] === 'hidden' ){
                continue;
            }
            $label = '<span class="estateMetabox__label"><i class="estateMetabox__label_icon fas ' . $group['icon'] . '"></i>%s</span>';
            add_meta_box(
                self::$postTypeMetaPrefix . '_'  . $group['id'],
                sprintf($label, $group['label']),
                function($post) use ($group) {
                    return $this->wpisMetaboxBody($post, $group['id']);
                },
                self::$postTypeName,
                'side',
                'low',
                ['__back_compat_meta_box' => false]
            );
        }

        // Gallery metabox
        $label = '<span class="estateMetabox__label"><i class="estateMetabox__label_icon fas fa-images"></i>%s</span>';
        add_meta_box(
            self::$postTypeMetaPrefix . '_'  . 'pictures',
            sprintf($label, 'Images du bien'),
            [$this, 'wpisMetaboxBodyPictures'],
            self::$postTypeName,
            'normal',
            'high'
        );
    }

    public function wpisMetaboxBodyPictures( $post ) {
        $medias = get_attached_media('image', $post->ID);

        echo '<ul id="estateMetaboxPictures" class="estateMetabox__pictures">';
        foreach ($medias as $media) {
            echo '<li>';
            echo '  <figure>';
            echo wp_get_attachment_image($media->ID);
            echo '  <i class="fa fa-times"></i>';
            echo '  </figure>';
            echo '</li>';
        }
        echo '</ul>';
        echo "<script>jQuery(function($){

            // Set all variables to be used in scope
            var frame,
                metaBox = $('#estateMetaboxPictures'), // Your meta box id here
                addImgLink = metaBox.find('.upload-custom-img'),
                delImgLink = metaBox.find( '.delete-custom-img'),
                imgContainer = $('#estateMetaboxPictures'),
                imgIdInput = metaBox.find( '.custom-img-id' );

            // ADD IMAGE LINK
            addImgLink.on( 'click', function( event ){

              event.preventDefault();

              // If the media frame already exists, reopen it.
              if ( frame ) {
                frame.open();
                return;
              }

              // Create a new media frame
              frame = wp.media({
                title: 'Select or Upload Media Of Your Chosen Persuasion',
                button: {
                  text: 'Use this media'
                },
                multiple: false  // Set to true to allow multiple files to be selected
              });


              // When an image is selected in the media frame...
              frame.on( 'select', function() {

                // Get media attachment details from the frame state
                var attachment = frame.state().get('selection').first().toJSON();

                // Send the attachment URL to our custom image input field.
                imgContainer.append( '<img src=\"'+attachment.url+'\" style=\"max-width:100%;\"/>' );

                // Send the attachment id to our hidden input
                imgIdInput.val( attachment.id );

                // Hide the add image link
                addImgLink.addClass( 'hidden' );

                // Unhide the remove image link
                delImgLink.removeClass( 'hidden' );
              });

              // Finally, open the modal on click
              frame.open();
            });


            // DELETE IMAGE LINK
            delImgLink.on( 'click', function( event ){

              event.preventDefault();

              // Clear out the preview image
              imgContainer.html( '' );

              // Un-hide the add image link
              addImgLink.removeClass( 'hidden' );

              // Hide the delete image link
              delImgLink.addClass( 'hidden' );

              // Delete the image id from the hidden input
              imgIdInput.val( '' );

            });

          });</script>";
    }

    private function wpisMetaboxBody( $post, $groupId ){
        foreach(EstatePostMetas::$postTypeMetas as $group) {
            if( $group['id'] === $groupId ){
                foreach($group['fields'] as $field){

                    $field = str_replace('.', '_', $field);

                    $value = get_post_meta($post->ID, self::$postTypeMetaPrefix . '_' . $field['field'], true);
                    $labelHtml = '<p class="post-attributes-label-wrapper"><label class="post-attributes-label" for="parent_id">' . $field['label'] . '</label></p>';

                    if( $field['type'] === 'readonly' ) {
                        echo $labelHtml;
                        echo '<input
                            style="width: 100%; border-radius: 3px; border: 1px solid #ddd; padding: 4px 8px; margin-bottom: 6px;"
                            value="' . $value .'"
                            disabled="disabled"
                            type="text"
                            name="' . self::$postTypeMetaPrefix . '_'  . $field['field'] . '"></input>';
                    }

                    if( $field['type'] === 'text' ) {
                        echo $labelHtml;
                        echo '<input
                            style="width: 100%; border-radius: 3px; border: 1px solid #ddd; padding: 4px 8px; margin-bottom: 6px;"
                            value="' . $value .'"
                            type="text"
                            name="' . self::$postTypeMetaPrefix . '_'  . $field['field'] . '"></input>';
                    }

                    if( $field['type'] === 'number' ) {
                        echo $labelHtml;
                        echo '<input
                            style="width: 100%; border-radius: 3px; border: 1px solid #ddd; padding: 4px 8px; margin-bottom: 6px;"
                            value="' . round($value) .'"
                            type="number"
                            name="' . self::$postTypeMetaPrefix . '_'  . $field['field'] . '"></input>';
                    }

                    if( $field['type'] === 'date' ) {
                        $value = substr($value, 0, 10);
                        echo $labelHtml;
                        echo '<input
                            style="width: 100%; border-radius: 3px; border: 1px solid #ddd; padding: 4px 8px; margin-bottom: 6px;"
                            value="' . $value .'"
                            type="date"
                            name="' . self::$postTypeMetaPrefix . '_'  . $field['field'] . '"></input>';
                    }

                    if( $field['type'] === 'boolean' ) {
                        echo '<section class="slider-checkbox">';
                        echo '  <input type="checkbox" ' . ($value ? 'checked' : '') . ' id="' . $field['field'] . '" name="' . self::$postTypeMetaPrefix . '_'  . $field['field'] . '" />';
                        echo '  <label class="label" for="' . $field['field'] . '">' . $field['label'] . '</label>';
                        echo '</section>';
                    }

                    if( $field['type'] === 'file-collection' ) {
                        foreach($value as $file) {
                            echo '<a href="' . $file->url . '" target="_blank">';
                            echo $file->filename . '.' . $file->format;
                            echo '</a>';
                        }
                    }

                    echo '<br>';
                }
            }
        }
    }

    public function wpisMetaboxSave( $post_id, $post ) {

        $post_type = get_post_type_object( $post->post_type );

        if ( !current_user_can( $post_type->cap->edit_post, $post_id ) )
            return $post_id;

        foreach(EstatePostMetas::$postTypeMetas as $group) {
            if( $group['id'] !== 'hidden' ){
                foreach($group['fields'] as $field){
                    if( !empty($_POST) ){
                        $meta = self::$postTypeMetaPrefix . '_' . $field['field'];
                        if( isset( $_POST[$meta] ) ) {
                            update_post_meta( $post_id, $meta, $_POST[$meta] );
                        }
                        else {
                            update_post_meta( $post_id, $meta, false );
                        }
                    }
                }
            }
        }
    }

    public function registerTaxonomies() {
        foreach( $this->taxonomies as $taxonomy ){

            $taxonomy['post_type'] = $this->customPostTypeName ?? self::$postTypeName;

            $labels = array(
                'name' => $taxonomy['plural_name'],
                'singular_name' => $taxonomy['single_name'],
                'search_items' =>  'Search ' . $taxonomy['plural_name'],
                'all_items' => 'All ' . $taxonomy['plural_name'],
                'parent_item' => 'Parent ' . $taxonomy['single_name'],
                'parent_item_colon' => 'Parent ' . $taxonomy['single_name'] . ':',
                'edit_item' => 'Edit ' . $taxonomy['single_name'],
                'update_item' => 'Update ' . $taxonomy['single_name'],
                'add_new_item' => 'Add New ' . $taxonomy['single_name'],
                'new_item_name' => 'New ' . $taxonomy['single_name'] . ' Name',
                'menu_name' => $taxonomy['plural_name']
            );

            $rewrite = isset( $taxonomy['rewrite'] ) ? $taxonomy['rewrite'] : array( 'slug' => $taxonomy['slug'] );
            $hierarchical = isset( $taxonomy['hierarchical'] ) ? $taxonomy['hierarchical'] : true;

            register_taxonomy( $taxonomy['slug'], $taxonomy['post_type'], array(
                'hierarchical' => $hierarchical,
                'labels' => $labels,
                'show_ui' => true,
                'query_var' => true,
                'rewrite' => $rewrite,
            ));
        }
    }

    static public function ucfirstUtf8($string){
        if($string == null || !is_string($string) ){
            return false;
        }
        $apresPremier = mb_substr($string,1); //mb_substr : Effectue une opération de type substr() basée sur le nombre de caractères
        $string = trim($string);
        $maj = utf8_decode($string);
        $maj = $maj[0];
        $a = 'ÀÁÂÃÄÅÆÇÈÉÊËÌÍÎÏÐÑÒÓÔÕÖØÙÚÛÜÝÞßàáâãäåæçèéêëìíîïðñòóôõöøùúûýýþÿ??';
        $b = 'aaaaaaaceeeeiiiidnoooooouuuuybsaaaaaaaceeeeiiiidnoooooouuuyybyRr';
        $maj = strtr($maj, utf8_decode($a), $b);
        $string = substr_replace($string, $maj, 0);
        $string = utf8_encode($string);
        $string = ucfirst($string);
        $string = $string.$apresPremier;
        return $string;
    }
}